 /*
 * Customizable Coin Weights - https://www.thingiverse.com/thing:2358931
 * by Dennis Hofmann - https://www.thingiverse.com/mightynozzle/about
 * created 2017-06-02
 * updated 2017-07-19
 *
 * Changelog
 * v1.1 - 2017-07-19:
 *     - added an option to choose which parts needed (box, lid or both)
 *     - added opening slot to remove the lid with a flathead screwdriver
 * 
 * This work is licensed under the Creative Commons - Attribution - Non-Commercial license.
 * https://creativecommons.org/licenses/by-nc/3.0/
 */

 // Parameter Section //
//-------------------//

/* [General Settings] */

// Choose, which part you want to see!
part = "all_parts__"; //[all_parts__:All Parts,box__:Box,lid__:Lid]

// Here you can set the coin diameter. A list of common coins (Europe first, sorry :-P): 1 Euro Cent: 16.25, 2 Cent: 18.75, 5 Cent: 21.25mm, 10 Cent: 19.75, 20 Cent: 22.25, 50 Cent: 24.25, 1 Euro : 23.25, 2 Euro: 25.75. 1 US Cent: 19.05, 5 US Cent: 21.209, 10 US Cent: 17.907, 25 US Cent: 24.257, 50 US Cent: 30.607, 1 US Dollar: 38.1/26.5 or 26.492. Default: 1 Euro Cent with 16.25mm
coin_diameter_in_millimeter = 18.75;

// Set the height of the coin chamber(s). This is the height without the bottom shell and without the lid height. E.g. 1/2/5 Euro Cent have a height of 1.67mm each. 20 Coins have a height of 33.4mm. 20x 5 Cent have a weight of 78.4g!
chamber_height_in_millimeter = 26.0;

// The number of coin chamber rows. The higher the number, the longer it takes to create the .stl-File. A healthy number will be between 1 and 4.
number_of_coin_chamber_rows = 2; //[1:15]

// The number of coin chamber columns. The higher the number, the longer it takes to create the .stl-File. A healthy number will be between 1 and 4.
number_of_coin_chamber_columns = 2; //[1:15] 

// A single slot on top of the box to remove the lid.
open_slot = "yes"; //[yes,no]

/* [Optional Settings] */
// Set an extra of Diameter of the coin chamber. Otherwise the coins may stuck. Default: 1.0
coin_diameter_extra_in_millimeter = 1.0;

// The shell thickness of the bottom, between coin chamber and box, and the lid height. Default: 2.0
shell_thickness_in_millimeter = 2.0;

/* [Hidden] */
shell_th = shell_thickness_in_millimeter;
coin_d = coin_diameter_in_millimeter;
coin_d_extra = coin_diameter_extra_in_millimeter;
chamber_h = chamber_height_in_millimeter;
chamber_rows = number_of_coin_chamber_rows;
chamber_columns = number_of_coin_chamber_columns;

cube_x = coin_d * number_of_coin_chamber_rows + coin_d_extra + shell_th * 2;
cube_y = coin_d * number_of_coin_chamber_columns + coin_d_extra + shell_th * 2;
cube_z = chamber_h + shell_th * 2 + 2.0;
cylinder_d = coin_d + coin_d_extra;
cylinder_h = chamber_h + shell_th * 2;
coin_hole_0_pos = -(coin_d + coin_d_extra) / 2 - shell_th;


 // Program Section //
//-----------------//

if(part == "box__") {
    box_part();
} else if(part == "lid__") {
    lid_part();
} else if(part == "all_parts__") {
    all_parts();
} else {
    all_parts();
}

 // Module Section //
//----------------//

module all_parts() {
    box_part();
    if(chamber_rows < chamber_columns) {
        translate([cube_x + 5.0, 0, 0]) {
            lid_part();
        }
    } else {
        translate([0, cube_y + 5.0, 0]) {
            lid_part();
        }
    }
}

module box_part() {
    color("DeepSkyBlue") box();
}

module lid_part() {
    color("HotPink") lid();
    
}

module box() {
    difference() {
        translate([coin_hole_0_pos, coin_hole_0_pos, 0]) {
            cube([cube_x, cube_y, cube_z]);
        }
        if(open_slot == "yes") {
            translate([-2, -cylinder_d / 2 - shell_th - 0.5, cylinder_h]) {
                cube([4.0, shell_th + 1, 2.1]);
            }
        }
        translate([0, 0, shell_th]) {
            for(row = [0 : chamber_rows - 1]) {
                for(column = [0 : chamber_columns - 1]) {
                    translate([row * coin_d, column * coin_d, 0]) {
                        cylinder(d = cylinder_d, h = cylinder_h + 0.1, $fn = 48);
                    }
                }
            }
        }
    }
}

module lid(){
    difference() {
        for(row = [0 : chamber_rows - 1]) {
            for(column = [0 : chamber_columns - 1]) {
                translate([row * coin_d, column * coin_d, 0]) {
                    cylinder(d = cylinder_d + shell_th * 2, h = shell_th, $fn = 50);
                    translate([0, 0, shell_th]) {
                        difference() {
                            cylinder(d = cylinder_d, h = 2, $fn = 48);
                            cylinder(d = cylinder_d - shell_th * 2, h = 2.1, $fn = 16);
                        }
                    }
                }
            }
        }
        translate([0, 0, shell_th]) {
            cube([(chamber_rows - 1) * coin_d, (chamber_columns - 1) * coin_d, 3]);
        }
    }
}
